
package jp.ksksue.app.bluetoothshabetter;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import jp.co.kayo.ykmjuku.andylib.component.BluetoothComponent;
import jp.co.kayo.ykmjuku.andylib.component.DeviceComponent.OnMessageReceiver;
import jp.co.kayo.ykmjuku.andylib.net.CommandMessage;
import jp.co.kayo.ykmjuku.andylib.tools.Logger;

public class SendMessages extends Fragment implements OnClickListener, OnMessageReceiver {
    /*
     * 以下、関連コードを区別しやすいよう、次のタグを入れています [AndyLib] AndyLibに関わるコード [UI]
     * ユーザインタフェースに関わるコード [AquesTalk] AquesTalkに関わるコード
     */
    static final private byte[] CR = Character.toString('\r').getBytes(); // [AquesTalk]
                                                                          // 改行コード

    private BluetoothComponent mBt; // [AndyLib] BluetoothComponent

    private EditText etMessage, etNumMessage; // [UI] 送信文字を入力する領域
    private Button btInit, btSend, btNumSend; // [UI] 文字を送信するときにタップするボタン
    private Spinner spJosushi; // [UI] 助数詞を選択するスピナー

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setHasOptionsMenu(true);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState) {

        // [UI] 各種UIのインスタンス生成
        View root = inflater.inflate(R.layout.chat_button, container, false);

        // 各ボタンのインスタンス取得
        btInit = (Button) root.findViewById(R.id.btInit);
        btSend = (Button) root.findViewById(R.id.btSend);
        btNumSend = (Button) root.findViewById(R.id.btNumSend);

        // 各ボタンをリスナー登録
        btInit.setOnClickListener(this);
        btSend.setOnClickListener(this);
        btNumSend.setOnClickListener(this);

        // 各エディットテキストのインスタンス取得
        etMessage = (EditText) root.findViewById(R.id.etMessage);
        etNumMessage = (EditText) root.findViewById(R.id.etNumMessage);

        // スピナーのインスタンス取得
        spJosushi = (Spinner) root.findViewById(R.id.spJosushi);

        return root;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();

        // [AndyLib] アプリが終了したらBluetoothComponentを終了させる
        if (mBt != null) {
            mBt.stop();
        }
    }

    /*
     * writeMessage [Bluetooth] String型の文字情報をBluetoothで送信
     */
    public void writeMessage(String str) {
        byte[] buf = new byte[256];

        if (str.length() < 256) {
            buf = str.toString().getBytes();
            if (mBt != null) {
                mBt.write(buf);
                mBt.write(CR); // [AquesTalk] 送信情報は改行で終える
            }
        }
    }

    /*
     * onClick [UI] Sendボタンをタップしたときの処理
     */
    public void onClick(View v) {
        if (v.getId() == R.id.btInit) {
            // 初期化コマンド「?」を送る
            writeMessage("?");
        } else if (v.getId() == R.id.btSend) {
            // EditTextの内容をそのまま送る
            writeMessage(etMessage.getText().toString());
        } else if (v.getId() == R.id.btNumSend) {
            // NUMKコマンドを作り送る
            StringBuilder str = new StringBuilder();
            str.append("<NUMK VAL=");
            str.append(etNumMessage.getText().toString());
            str.append(" COUNTER=");
            str.append(spJosushi.getSelectedItem().toString());
            str.append(">");
            writeMessage(str.toString());
        }
    }

    /*
     * [AndyLib] 以下AndyLibのサンプルコード jxttx-bt3 を参考 Bluetooth接続の状態が変化した場合、コールバックされる
     * 使用するには onResume で mBt.setOnMessageReceiver(this); と登録する
     */
    public void onReceive(CommandMessage msg) {

        // Bluetoothに変化（接続・送信・受信）があった場合に以下処理が実行される
        switch (msg.getCommand()) {

            case BluetoothComponent.MESSAGE_STATE_CHANGE:
                byte state = msg.getBuffer()[0];
                Logger.d("MESSAGE_STATE_CHANGE: " + state);

                switch (state) {

                // Bluetoothが接続された場合
                    case BluetoothComponent.STATE_CONNECTED:
                        Logger.d("STATE_CONNECTED");
                        if (getActivity() != null) {
                            Toast.makeText(getActivity(),
                                    "Connected",
                                    Toast.LENGTH_SHORT).show();
                        }
                        break;
                    case BluetoothComponent.STATE_CONNECTING:
                        Logger.d("STATE_CONNECTING");
                        break;
                    case BluetoothComponent.STATE_LISTEN:
                    case BluetoothComponent.STATE_NONE:
                        Logger.d("STATE_NOT_CONNECTED");
                        if (getActivity() != null) {
                            Toast.makeText(getActivity(),
                                    "Error",
                                    Toast.LENGTH_SHORT).show();
                        }
                        break;
                }
                break;

            // Bluetoothへメッセージを送信した場合
            case BluetoothComponent.MESSAGE_WRITE: {
                byte[] writeBuf = (byte[]) msg.getBuffer();
                StringBuffer buf = new StringBuffer();
                for (byte b : writeBuf) {
                    if (buf.length() > 0) {
                        buf.append(",");
                    }
                    buf.append(Integer.toHexString(b));
                }
                Logger.d("sent:" + buf.toString());
            }
                break;

            // Bluetoothからメッセージを受信した場合
            case BluetoothComponent.MESSAGE_READ: {
                byte[] readBuf = (byte[]) msg.getBuffer();
                // construct a string from the valid bytes in the buffer
                String readMessage = new String(readBuf);
                Logger.d("recv:" + readMessage);
            }
                break;
        }
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    @Override
    public void onResume() {
        super.onResume();
        if (mBt == null) {
            mBt = (BluetoothComponent) getActivity().getSupportFragmentManager()
                    .findFragmentByTag("BT");
            if (mBt != null) {
                mBt.setOnMessageReceiver(this); // コールバック関数 onReceive が登録される
            }
        }
    }

    /*
     * [AndyLib] 以上AndyLibサンプルコード jxttx-bt3 を参考
     */

}
