package jp.maruishi.android.remotecontrollcar03;

import jp.maruishi.android.remotecontrollcar03.R;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.Toast;

public class ControllerActivity extends Activity {

	private static final String TAG = "RemoteController";

	private BluetoothAdapter mBluetoothAdapter = null;
	private BluetoothService mBluetoothService = null;

	private static final int REQUEST_CONNECT_DEVICE_SECURE = 1;
	private static final int REQUEST_CONNECT_DEVICE_INSECURE = 2;
	private static final int REQUEST_ENABLE_BT = 3;

	public static final int MESSAGE_STATE_CHANGE = 1;
	public static final int MESSAGE_READ = 2;
	public static final int MESSAGE_WRITE = 3;
	public static final int MESSAGE_DEVICE_NAME = 4;
	public static final int MESSAGE_TOAST = 5;

	public static final String TOAST = "toast";
	public static final String DEVICE_NAME = "device_name";

	private CarController mCarController;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		Log.d(TAG, "onCreate");

		mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

		if (mBluetoothAdapter == null) {
			Toast.makeText(this, "Bluetooth is not available",
					Toast.LENGTH_LONG).show();
			finish();
			return;
		}

		mCarController = (CarController) findViewById(R.id.carController);
		mCarController.setNotifyInterval(50);
		mCarController.setOnControllerChanged(mControllerInterface);

	}

	ControllerInterface mControllerInterface = new ControllerInterface() {
		@Override
		public void onControllerChanged(int x, int y) {
			// Log.d(TAG, "onControllerChanged" + x + "," + y +
			// "**********************************************");
			sendCommand(x, y);
		}
	};

	private void sendCommand(int x, int y) {
		byte[] transmitData = new byte[5];

		if ((x == -1) || (y == -1)) {
			// ブレーキをかける
			
			// コマンドヘッダ部 B = コマンドセット = バイナリ
			transmitData[0] = 'B';
			// 左右モータの方向 1 = 正転,0 = 逆転
			transmitData[1] = (byte) 1;
			transmitData[2] = (byte) 1;
			// 左右モータのパワー 0〜255
			transmitData[3] = (byte) 0;
			transmitData[4] = (byte) 0;

		} else {
			int motorPower_R;
			int motorPower_L;

			motorPower_R = convertPower(x, y);
			motorPower_L = convertPower(10 - x, y);
			Log.d(TAG,"motorPower_R = " + motorPower_R + "motorPower_L = " + motorPower_L);
			// コマンドヘッダ部 B = コマンドセット = バイナリ
			transmitData[0] = 'B';
			// 左右モータの方向 1 = 正転,0 = 逆転
			transmitData[1] = (byte) ((motorPower_R >= 0) ? 1 : 0);
			transmitData[2] = (byte) ((motorPower_L >= 0) ? 1 : 0);
			// 左右モータのパワー 0〜255
			transmitData[3] = (byte) Math.abs(motorPower_R);
			transmitData[4] = (byte) Math.abs(motorPower_L);
		}
		// Bluetoothからデータ送信
		mBluetoothService.write(transmitData);
	}

	private int convertPower(int x, int y) {
		int unit = 255 / 5;
		int mapx = -5 + x;
		int mapy =  5 - y;
		int power;

		//Log.d(TAG,"map x,y = "+mapx+","+mapy);
		
		if (mapx >= 0) {
			//Log.d(TAG,"Zone");
			power = unit * mapy;
		} else {
			power = ((unit * x) / 5) * mapy;
		}
		return power;
	}

	@Override
	public synchronized void onResume() {
		super.onResume();

		// Performing this check in onResume() covers the case in which BT was
		// not enabled during onStart(), so we were paused to enable it...
		// onResume() will be called when ACTION_REQUEST_ENABLE activity
		// returns.
		if (mBluetoothService != null) {
			// Only if the state is STATE_NONE, do we know that we haven't
			// started already
			if (mBluetoothService.getState() == BluetoothService.STATE_NONE) {
				// Start the Bluetooth chat services
				Log.d(TAG, "Start Bluetooth connection service");
				mBluetoothService.start();
			}
		}
	}

	@Override
	public void onStart() {
		super.onStart();
		if (!mBluetoothAdapter.isEnabled()) {

			Intent enableIntent = new Intent(
					BluetoothAdapter.ACTION_REQUEST_ENABLE);
			startActivityForResult(enableIntent, REQUEST_ENABLE_BT);

		} else {
			if (mBluetoothService == null) {
				setupConnection();
			}
		}
	}

	@Override
	public synchronized void onPause() {
		super.onPause();
	}

	@Override
	public void onStop() {
		super.onStop();
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		if (mBluetoothService != null) {
			mBluetoothService.stop();
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.option_menu, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		Intent serverIntent = null;
		switch (item.getItemId()) {
		case R.id.secure_connect_scan:
			// Launch the DeviceListActivity to see devices and do scan
			serverIntent = new Intent(this, DeviceListActivity.class);
			startActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE_SECURE);
			return true;
		case R.id.insecure_connect_scan:
			// Launch the DeviceListActivity to see devices and do scan
			serverIntent = new Intent(this, DeviceListActivity.class);
			startActivityForResult(serverIntent,
					REQUEST_CONNECT_DEVICE_INSECURE);
			return true;
		default:
			Log.d(TAG, "Unknown menu ID");
			break;
		}
		return false;
	}

	// The Handler that gets information back from the BluetoothChatService
	private final Handler mHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case MESSAGE_STATE_CHANGE:
				switch (msg.arg1) {
				case BluetoothService.STATE_CONNECTED:
					Toast.makeText(getApplicationContext(), "STATE_CONNECTED",
							Toast.LENGTH_SHORT).show();
					sendCommand(-1, -1);

					break;
				case BluetoothService.STATE_CONNECTING:
					Toast.makeText(getApplicationContext(), "STATE_CONNECTING",
							Toast.LENGTH_SHORT).show();
					break;
				case BluetoothService.STATE_LISTEN:
					Toast.makeText(getApplicationContext(), "STATE_LISTEN",
							Toast.LENGTH_SHORT).show();
				case BluetoothService.STATE_NONE:
					Toast.makeText(getApplicationContext(), "STATE_NONE",
							Toast.LENGTH_SHORT).show();
					break;
				}
				break;
			case MESSAGE_WRITE:
				break;
			case MESSAGE_READ:
				break;
			case MESSAGE_DEVICE_NAME:
				Toast.makeText(getApplicationContext(), "MESSAGE_DEVICE_NAME",
						Toast.LENGTH_SHORT).show();
				break;
			case MESSAGE_TOAST:
				Toast.makeText(getApplicationContext(),
						msg.getData().getString(TOAST), Toast.LENGTH_SHORT)
						.show();
				break;
			}
		}
	};

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		Log.d(TAG, "onActivityResult " + resultCode);

		switch (requestCode) {
		case REQUEST_ENABLE_BT:
			if (resultCode == Activity.RESULT_OK) {
				// Bluetooth is now enabled, so set up a chat session
				setupConnection();
			} else {
				// User did not enable Bluetooth or an error occured
				Log.d(TAG, "BT not enabled");
				Toast.makeText(this, "BT", Toast.LENGTH_SHORT).show();
				finish();
			}
			break;
		case REQUEST_CONNECT_DEVICE_SECURE:
			// When DeviceListActivity returns with a device to connect
			if (resultCode == Activity.RESULT_OK) {
				connectDevice(data, true);
			}
			break;
		case REQUEST_CONNECT_DEVICE_INSECURE:
			// When DeviceListActivity returns with a device to connect
			if (resultCode == Activity.RESULT_OK) {
				connectDevice(data, false);
			}
			break;

		default:
			break;
		}
	}

	private void connectDevice(Intent data, boolean secure) {
		// Get the device MAC address
		String address = data.getExtras().getString(
				DeviceListActivity.EXTRA_DEVICE_ADDRESS);
		// Get the BLuetoothDevice object
		BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
		// Attempt to connect to the device
		mBluetoothService.connect(device, secure);
	}

	private void setupConnection() {

		// Initialize the BluetoothChatService to perform bluetooth connections
		mBluetoothService = new BluetoothService(this, mHandler);

	}

}